#!/usr/bin/env python

""" 
 Web service that generates lists of cells, areas etc. Should produce geo-rss, wfs-xml 
 or other machine readable formats.

 Part of the GLoc visualization/debug backend.

 Accessed by mod_python to render web pages.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import psycopg2, psycopg2.extras, pylib.wfs as  wfs, pylib.georss as georss, pylib.glocdb as glocdb

def index(req,uncache=None):
    """
    Give users who try to access the service without any parameters a nasty message.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param req: HTTP request
    @type req: automatic
    @param uncache: Not used at the moment

    @return: nasty message
    @rtype: string
    """

    req.content_type = 'text/plain'
    return "Nothing to see here. Go away!"


def getcellarea(req, cid, type=None, areal=False):
    """
    Returns a wfs-xml list of the convex hull or a point list of our observations of a single cell. 

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    
    @param req: HTTP request
    @type req: automatic
    @param cid: ID of requested cell
    @type cid: int
    @param type: limit to a given signal level
        - 0 rxlev > 90
        - 1 rxlev < 90 > 80
        - 2 rxlev < 80
    @type type: int
    @param areal: if true the returned list is a filled polygon of the convex hull of the observations, if false a point list
    @type areal: boolean

    @return: wfs-xml
    @rtype: string

    @todo: Need to include name in call to addfilledpoly()
    """

    collection = glocdb.getcellobservations(cid,type)

    wfsobj = wfs.WFS("http://%s%s"%(req.hostname, req.uri))
    wfsobj.addboundedby(glocdb.getbounds(collection))

    points = []
    
    for entry in collection:
        points.append([entry['longitude'],entry['latitude']])
        if not areal: wfsobj.addpoint(entry['longitude'],entry['latitude'])

    if areal:
        (ul,ll) = glocdb.hulls(points)
        wfsobj.addfilledpoly(ul,ll,glocdb.getbounds(collection),"TODO NAME HERE")        

    return str(wfsobj)

def getcells(req, mcc=None, mnc=None, la=None):
    """
    Returns a geo-rss list of unique cells in the database.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param req: HTTP request
    @type req: automatic
    @param mcc: limit the returned list to a given Mobile Content Code.
    @type mcc: int
    @param mnc: limit the returned list to a given Mobile Network Code. (Make no sense if mcc is None)
    @type mnc: int
    @param la: limit the returned list to a given Location Area. (Makes no sense if mnc is None)
    @type la: int

    @return: geo-rss
    @rtype: str
    """

    (titlestr,collection) = glocdb.getcells(mcc,mnc,la)

    geoobj = georss.GeoRss("http://%s%s"%(req.hostname, req.uri),"Brendan Johan Lee","Gloc collected cells %s"%titlestr)

    for entry in collection:
        summary = "MCC: %i&lt;br /&gt;MNC: %i&lt;br /&gt;LA: %i&lt;br /&gt;CELLID: %i"%(entry['mcc'],entry['mnc'],entry['la'],entry['cellid'])
        geoobj.addentry(entry['name'],"/maphandler.py/singlecell?cid=%i"%entry['cid'],entry['cid'],entry['longitude'],entry['latitude'],summary)

    return str(geoobj)

def getnorwegian(req):
    collection = glocdb.getnorwegian()

    geoobj = georss.GeoRss("http://%s%s"%(req.hostname, req.uri),"Brendan Johan Lee","Gloc ripped norwegiancells")

    for entry in collection:
        #TODO: If this is to be used, we ned to modify the database class so that we get the information about the site
        #summary = "MCC: %i&lt;br /&gt;MNC: %i&lt;br /&gt;LA: %i&lt;br /&gt;CELLID: %i"%(entry['mcc'],entry['mnc'],entry['la'],entry['cellid'])
        geoobj.addentry(entry['sid'],"",0,entry['longitude'],entry['latitude'],"")

    return str(geoobj)
