#!/usr/bin/env python

""" 
 Web service that generates maps with cells rendered on them. Generates HTML-pages.

 Part of the GLoc visualization/debug backend.

 Accessed by mod_python to render web pages.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import pylib.map as map, pylib.mappage as mappage, pylib.glocdb as glocdb

def index(req,uncache=None):
    """
    Give users who try to access the service without any parameters a nasty message.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param req: HTTP request
    @type req: automatic
    @param uncache: Not used at the moment

    @return: nasty message
    @rtype: string
    """

    req.content_type = 'text/plain'
    return "Nothing to see here. Go away!"

def overview(req, mcc=None, mnc=None, la=None):
    """ 
    Generate an overview map of all of our cells in our database

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param req: HTTP request
    @type req: automatic
    @param mcc: only show cells from given Mobile Country Code
    @type mcc: int
    @param mnc: only show cells from given Mobile Network Code (makes no sense if mcc is None)
    @type mnc: int
    @param la: only show cells from given Location Area (makes no sense if mnc is None)
    @type la: int

    @return: map as HTML page
    @rtype: string
    @todo: Implement mcc, mnc and la filtering
    """

    body = """<h1 id="title" style="position:absolute;top:3px;left:50px;z-index:1000;">SHOWING ALL OBSERVED CELLS</h1>
<p id="shortdesc" style="position:absolute;top:35px;left:70px;z-index:1000;">
\tShows all cells in database, loading could be slow!
</p>"""
    page = mappage.defaultpage('map',body)
    cellmap = map.defaultMap('map','map')
    icon = cellmap.addicon(map.Icon("/images/icons/peak.png",map.Size(38,38),map.Pixel(0,-38)));
    cellayer = map.Layer("GeoRSS","Cells","/cellhandler.py/getcells")
    cellayer.addoption(map.LayerOption("icon",icon))
    cellayer.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))
    cellmap.addlayer(cellayer)
    #TODO: When implementing mcc, mnc and la filtering, this too needs to be updates
    autozoom = cellmap.addbounds(map.Bounds(glocdb.getbounds(glocdb.getcells()[1]),map.Projection("EPSG:4326"),map.Projection("EPSG:900913")))
    cellmap.setautozoom(autozoom)

    page.addmap(cellmap,"width:100%;height:100%;")

    return str(page)

def singlecell(req, cid):
    """ 
    Generate a map showing details of a single cell in our database

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param req: HTTP request
    @type req: automatic
    @param cid: ID of cell to show details on
    @type cid: int

    @return: map as HTML page
    @rtype: string
    @todo: Should do some database magic here to get name of cell et al
    """

    body = """<h1 id="title" style="position:absolute;top:3px;left:50px;z-index:1000;">CELL DETAILS FOR CELL: TODO TODO TODO</h1>
<p id="shortdesc" style="position:absolute;top:35px;left:70px;z-index:1000;">
\tShows all observations of a cell at different strenghts. Use layer menu for visual types!
</p>"""
    page = mappage.defaultpage('Cell details',body)
    cellmap = map.defaultMap('map','map')

    greenstyle = map.Style()
    greenstyle.addoption(map.LayerOption("fillColor","'#00FF00'"))
    greenstyle.addoption(map.LayerOption("fillOpacity","0.2"))
    greenstyle.addoption(map.LayerOption("strokeColor","'#00FF00'"))
    greenstyle.addoption(map.LayerOption("strokeWidth",1))
    greenstyle.addoption(map.LayerOption("strokeLinecap","'round'"))
    greenstyle.addoption(map.LayerOption("strokeDashstyle","'solid'"))
    greenstyle.addoption(map.LayerOption("pointRadius",3))
    gs = cellmap.addstyle(greenstyle)

    greenstylemap = map.Stylemap()
    greenstylemap.addoption(map.LayerOption("default",gs))
    gsm = cellmap.addstylemap(greenstylemap)

    strongcells = map.Layer("GML","Strong cellobservations","/cellhandler.py/getcellarea?cid=%s&type=2"%cid)
    strongcells.addoption(map.LayerOption("styleMap",gsm))
    strongcells.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    strongcellspoly = map.Layer("GML","Strong cells areal (convex hull)","/cellhandler.py/getcellarea?cid=%s&type=2&areal=true"%cid)
    strongcellspoly.addoption(map.LayerOption("styleMap",gsm))
    strongcellspoly.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    yellowstyle = map.Style()
    yellowstyle.addoption(map.LayerOption("fillColor","'#FFFF00'"))
    yellowstyle.addoption(map.LayerOption("fillOpacity","0.2"))
    yellowstyle.addoption(map.LayerOption("strokeColor","'#FFFF00'"))
    yellowstyle.addoption(map.LayerOption("strokeWidth",1))
    yellowstyle.addoption(map.LayerOption("strokeLinecap","'round'"))
    yellowstyle.addoption(map.LayerOption("strokeDashstyle","'solid'"))
    yellowstyle.addoption(map.LayerOption("pointRadius",3))
    ys = cellmap.addstyle(yellowstyle)

    yellowstylemap = map.Stylemap()
    yellowstylemap.addoption(map.LayerOption("default",ys))
    ysm = cellmap.addstylemap(yellowstylemap)

    mediumcells = map.Layer("GML","Medium cellobservations","/cellhandler.py/getcellarea?cid=%s&type=1"%cid)
    mediumcells.addoption(map.LayerOption("styleMap",ysm))
    mediumcells.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    mediumcellspoly = map.Layer("GML","Medium cells areal (convex hull)","/cellhandler.py/getcellarea?cid=%s&type=1&areal=true"%cid)
    mediumcellspoly.addoption(map.LayerOption("styleMap",ysm))
    mediumcellspoly.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    redstyle = map.Style()
    redstyle.addoption(map.LayerOption("fillColor","'#FF0000'"))
    redstyle.addoption(map.LayerOption("fillOpacity","0.2"))
    redstyle.addoption(map.LayerOption("strokeColor","'#FF0000'"))
    redstyle.addoption(map.LayerOption("strokeWidth",1))
    redstyle.addoption(map.LayerOption("strokeLinecap","'round'"))
    redstyle.addoption(map.LayerOption("strokeDashstyle","'solid'"))
    redstyle.addoption(map.LayerOption("pointRadius",3))
    rs = cellmap.addstyle(redstyle)

    redstylemap = map.Stylemap()
    redstylemap.addoption(map.LayerOption("default",rs))
    rsm = cellmap.addstylemap(redstylemap)

    weakcells = map.Layer("GML","Weak cellobservations","/cellhandler.py/getcellarea?cid=%s&type=0"%cid)
    weakcells.addoption(map.LayerOption("styleMap",rsm))
    weakcells.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    weakcellspoly = map.Layer("GML","Weak cells areal (convex hull)","/cellhandler.py/getcellarea?cid=%s&type=0&areal=true"%cid)
    weakcellspoly.addoption(map.LayerOption("styleMap",rsm))
    weakcellspoly.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))

    cellmap.addlayer(weakcellspoly)
    cellmap.addlayer(weakcells)
    cellmap.addlayer(mediumcellspoly)
    cellmap.addlayer(mediumcells)
    cellmap.addlayer(strongcellspoly)
    cellmap.addlayer(strongcells)

    autozoom = cellmap.addbounds(map.Bounds(glocdb.getbounds(glocdb.getcellobservations(cid)),map.Projection("EPSG:4326"),map.Projection("EPSG:900913")))
    cellmap.setautozoom(autozoom)

    page.addmap(cellmap,"width:100%;height:100%;")
    return str(page)

def norwegian(req):

    body = """<h1 id="title" style="position:absolute;top:3px;left:50px;z-index:1000;">SHOWING ALL OBSERVED CELLS</h1>
<p id="shortdesc" style="position:absolute;top:35px;left:70px;z-index:1000;">
\tShows all cells in database, loading could be slow!
</p>"""
    page = mappage.defaultpage('map',body)
    cellmap = map.defaultMap('map','map')
    icon = cellmap.addicon(map.Icon("/images/icons/peak.png",map.Size(38,38),map.Pixel(0,-38)));
    cellayer = map.Layer("GeoRSS","Cells","/cellhandler.py/getnorwegian")
    cellayer.addoption(map.LayerOption("icon",icon))
    cellayer.addoption(map.LayerOption("projection",map.Projection("EPSG:4326")))
    cellmap.addlayer(cellayer)
    autozoom = cellmap.addbounds(map.Bounds(glocdb.getbounds(glocdb.getnorwegian()),map.Projection("EPSG:4326"),map.Projection("EPSG:900913")))
    cellmap.setautozoom(autozoom)

    page.addmap(cellmap,"width:100%;height:100%;")

    return str(page)
