#!/usr/bin/env python

""" 
 Represents a geo-rss file containing any number of single points.
 Used when generating overviews of cells. When generating single
 points for cell observations, please use WFS instead.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import datetime
from time import strftime

class GeoRss:
    """ 
    Represents a geo-rss file containing any given number of 
    points.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, site, author, title):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param site: Base URL of website
        @type site: string
        @param author: Name of author
        @type author: string
        @param title: Title of file
        @type title: string
        """

        self.items = []
        year = datetime.date.today().year
        updated = strftime("%Y-%m-%dT%H:%M:%S-01:00")
        self.header = """<?xml version="1.0" encoding="utf-8"?>
<feed xmlns="http://www.w3.org/2005/Atom" xmlns:geo="http://www.w3.org/2003/01/geo/wgs84_pos#"> 
	<rights>Copyright %s %s</rights> 
	<title>%s</title>
	<link href="%s"/>
	<updated>%s</updated>
	<author>
		<name>%s</name>
	</author>\n"""%(year,author,title,site,updated,author)

    def addentry(self,title,link,id,longitude,latitude,summary):
        """ 
        Add an entry to this geo-rss file. 

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param title: Title of the point
        @type title: string
        @param link: URL that the point should link to
        @type link: string
        @param id: ID of this point
        @type id: int
        @param longitude: longitude of point
        @type longitude: string
        @param latitude: latitude of point
        @type latitude: string
        @param summary: HTML-summary describing the point
        @type summary: string
        """
        updated = strftime("%Y-%m-%dT%H:%M:%S-01:00")
        self.items.append("""<entry>
		<title>%s</title>
		<link rel="alternate" type="text/html" href="%s" />
		<id>%s</id>
		<updated>%s</updated>
		<summary type="html">
		%s
		</summary>
		<geo:long>%s</geo:long>	
		<geo:lat>%s</geo:lat>
</entry>\n"""%(title,link,id,updated,summary,longitude,latitude))

    def __str__(self):
        """ 
        Returns a textual representation of a geo-rss file

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: geo-rss file as text
        @rtype: string
        """
        
        retval = self.header
        for item in self.items:
            retval += item
        retval += "</feed>"
        return retval
