#!/usr/bin/env python

""" 
 Module that interfaces with the GLoc main database.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""


import psycopg2, psycopg2.extras

#: Diferent rxlevel limits used for limiting searches
rxlevlimits = ["rxlev > 90","rxlev < 90 AND rxlev > 80","rxlev < 80"]

def getcellobservations(cid,type=None):
    """ 
    Returns a database cursor containing all observations of a given cell.
    If the optional 

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param cid: ID of cell to return observations of
    @type cid: int
    @param type: limit to a given signal level
        - 0 rxlev > 90
        - 1 rxlev < 90 > 80
        - 2 rxlev < 80
    @type type: int

    @return: all results as tupples
    @rtype: list of tupples
    """

    cur = __getdbcursor__()
    cur.execute("SELECT * FROM gatheredcellobservations WHERE cid=%i %s"%(int(cid),"AND %s"%rxlevlimits[int(type)] if type else ""))
    return cur.fetchall()

def getcells(mcc=None,mnc=None,la=None):
    """ 
    Return all of the cells in the GLoc database. Can optionally be
    limited to cells within a Mobile Country Code, Mobile Network
    Code and Location Area

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param mcc: Optional limit on Mobile Country Code
    @type mcc: int
    @param mnc: Optional limit on Mobile Network Code (makes no sense if mcc is None)
    @type mnc: int
    @param la: Optional limit on Location Area (makes no sense if mnc is None)
    @type la: int

    @return: Textual representation of optional limitations and information on cells
    @rtype: tupple of string and list of tupples
    """

    titlestr = sql =""
    first = True
    if mcc:
        sql += "%s mcc=%i "%("WHERE" if first else "AND",int(mcc))
        titlestr += "mcc=%i "%(int(mcc))
        first = False
    if mnc:
        sql += "%s mnc=%i "%("WHERE" if first else "AND",int(mnc))
        titlestr += "mnc=%i "%(int(mnc))
        first = False
    if la:
        sql += "%s la=%i "%("WHERE" if first else "AND",int(la))
        titlestr += "la=%i "%(int(la))
        first = False

    cur = __getdbcursor__()
    cur.execute("SELECT * FROM gatheredcells %s"%(sql))
    return (titlestr, cur.fetchall())

def getnorwegian():
    cur = __getdbcursor__()
    cur.execute("SELECT * FROM norwegiancells.sites;")
    return cur.fetchall()

def getbounds(points):
    """ 
    Calculates boundaries around a set of points given by
    latitude and longitude. In other words, calculate the smallest box
    that surrounds all of the points.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param points: points to calculate boundaries around
    @type points: list of tupples containing at least latitude and longitude (eg. return value of getcellobservations())

    @return: (longitudemin, latitudemin, longitudemax, latitudemax)
    @rtype: tupple
    @todo: Should be moved to an algorithm module
    """

    latmin = lonmin = 1000
    latmax = lonmax = 0
    for entry in points:
        if entry['latitude'] < latmin: latmin = entry['latitude']
        if entry['latitude'] > latmax: latmax = entry['latitude']
        if entry['longitude'] < lonmin: lonmin = entry['longitude']
        if entry['longitude'] > lonmax: lonmax = entry['longitude']
    return (lonmin, latmin, lonmax, latmax)

def orientation(p,q,r):
    """ 
       Return positive if p-q-r are clockwise, neg if ccw, zero if colinear.

       @author: Brendan Johan Lee
       @contact: brendajl@simula.no

       @version: 1.0

       @param p: p 
       @type p: point
       @param q: q
       @type q: point
       @param r: r
       @type r: point

       @return: neg if ccw, pos if cw, 0 if colinear
       @rtype: int
       @todo: Should be moved to an algorithm module
       """

    return (q[1]-p[1])*(r[0]-p[0]) - (q[0]-p[0])*(r[1]-p[1])

def hulls(Points):
    """ 
    Graham scan to find upper and lower convex hulls of a set of 2d points.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param Points: Points to find convex hull of
    @type Points: Points

    @return: Upper and lower convex hulls
    @rtype: tupple of lists
    @todo: Should be moved to an algorithm module
    """

    U = []
    L = []
    Points.sort()
    for p in Points:
        while len(U) > 1 and orientation(U[-2],U[-1],p) <= 0: U.pop()
        while len(L) > 1 and orientation(L[-2],L[-1],p) >= 0: L.pop()
        U.append(p)
        L.append(p)
    return U,L

def __getdbcursor__():
    """ 
    Connect to the GLoc database and return the connection object.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @return: Connetion to database
    @rtype: psycopg2.extras.DictConnection
    @todo: dbname, user, password and hosts need to be moved into a separate file ASAP
    """

    conn = psycopg2.extras.DictConnection("dbname='gloc' user='gloc' password='99ab97a' host='localhost'")
    return conn.cursor()
