#!/usr/bin/env python
""" 
 Represents an openlayers map. This class is essentially a (as of now incomplete)
 wrapper around the openlayers javascript API.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""


class Map:
    """ 
    Represents an openlayers map

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self, name, div):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param name: Name of javascript map object
        @type name: string
        @param div: ID of div-tag to render map on
        @type div: string
        """

        self.name = name
        self.div = div
        self.layers = []
        self.controls = []
        self.options = []
        self.icons = []
        self.styles = []
        self.stylemaps = []
        self.bounds = []
        self.autozoom = None
        self.layercounter = self.controlcounter = self.iconcounter = self.stylecounter = self.stylemapcounter = self.boundcounter = 0

    def addoption(self, option):
        """ 
        Add an option (projection, units, etc.) to the map

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param option: Option to add to map
        @type option: map.MapOption
        """

        self.options.append(option)

    def addbounds(self,bounds):
        """ 
        Add a boundary to the map

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param bounds: Boundary to add to map
        @type bounds: Bounds

        @return: A textual name of the javascript bounds object
        @rtype: string
        """

        self.bounds.append(bounds)
        self.boundcounter += 1
        bounds.setid(self.boundcounter)
        return "bounds%i"%self.boundcounter

    def setautozoom(self,autozoom):
        """ 
        Connect an autozoom to the map. In other words a Bounds
        object that the map will zoom to after loading.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param autozoom: Boundary to zoom map to after loading
        @type autozoom: map.Bounds
        """

        self.autozoom = autozoom

    def addstylemap(self,stylemap):
        """ 
        Add a stylemap to the map.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param stylemap: Stylemap to add to the map
        @type stylemap: map.Stylemap

        @return: A textual name of the javascript stylemap object
        @rtype: string
        """

        self.stylemaps.append(stylemap)
        self.stylemapcounter += 1
        stylemap.setid(self.stylemapcounter)
        return "stylemap%i"%self.stylemapcounter

    def addstyle(self,style):
        """ 
        Add a style to the map.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param style: Style to add to the map
        @type style: map.Style

        @return: A textual name of the javascript style object
        @rtype: string
        """

        self.styles.append(style)
        self.stylecounter += 1
        style.setid(self.stylecounter)
        return "style%i"%self.stylecounter
        
    def addicon(self,icon):
        """ 
        Add an icon to the map. This method adds the icon as a
        resource to the map, it does not add a visible icon.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param icon: Icon resource to be added
        @type icon: map.Icon

        @return: A textual name of the javascript icon object
        @rtype: string
        """

        self.icons.append(icon)
        self.iconcounter += 1
        icon.setid(self.iconcounter)
        return "icon%i"%self.iconcounter

    def addlayer(self, layer):
        """ 
        Add a layer to the map. 

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param layer: Layer to be added to the map
        @type layer: map.Layer

        @return: A textual name of the javascript layer object
        @rtype: string
        """

        self.layers.append(layer)
        self.layercounter += 1
        layer.setid(self.layercounter)
        return "layer%i"%self.layercounter

    def addcontrol(self,control):
        """ 
        Add a control to the map.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param control: Control to be added to the map
        @type control: map.Control

        @return: A textual name of the javascript control object
        @rtype: string
        @todo: Remove duplicate controls
        """

        self.controls.append(control)
        self.controlcounter += 1
        control.setid(self.controlcounter)

    def setdefaults(self):
        raise NotImplementedError

    def __str__(self):
        """ 
        Return a textual representation of the map. In other words
        return the javascript code representing this map.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of map
        @rtype: string
        """

        retval = "%s"%"\n".join([str(x) for x in self.bounds])
        retval += "\n\t\t\tvar %s = new OpenLayers.Map('%s',{%s\n\t\t\t});\n"%(self.name,self.div,','.join(["\n\t\t%s"%str(x) for x in self.options]))
        retval += "%s\n"%"\n".join([str(x) for x in self.icons])
        retval += "%s\n"%"\n".join([str(x) for x in self.styles])
        retval += "%s\n"%"\n".join([str(x) for x in self.stylemaps])
        retval += "".join([str(x) for x in self.layers])
        retval += "\t\t\t%s.addLayers([%s]);\n"%(self.name,",".join(["layer%i"%x.id for x in self.layers]))
        retval += "".join(["\t\t\t%s.addControl(%s);\n"%(self.name,str(x)) for x in self.controls])
        if self.autozoom: retval += "\t\t\t%s.zoomToExtent(%s);"%(self.name,self.autozoom)
        else: retval += "\t\t\tif (!%s.getCenter()) {%s.zoomToMaxExtent()}"%(self.name,self.name)
        return retval

class Stylemap:
    """ 
    Wrapper around the OpenLayers StyleMap object. One StyleMap object
    can contain several Styles.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """

        self.id = 0
        self.options = []

    def addoption(self,option):
        """ 
        Add an option to this stylemap

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param option: Option to be added to stylemap
        @type option: map.LayerOption
        """

        self.options.append(option)

    def setid(self,id):
        """ 
        Set the id of this stylemap

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: id to assign to this stylemap
        @type id: int
        """

        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the stylemap. In other words
        return the javascript code representing the stylemap.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of stylemap
        @rtype: string
        """

        return "\t\t\tvar stylemap%i = new OpenLayers.StyleMap({%s\n\t\t\t});\n"%(self.id,','.join(["\n\t\t\t\t%s"%str(x) for x in self.options]) if len(self.options)>0 else "")

class Style:
    """ 
    Wrapper around the OpenLayers Style object.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """

        self.id = 0
        self.options = []

    def addoption(self,option):
        """ 
        Add an option to this style.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param option: Option to be added to style
        @type option: map.LayerOption
        """

        self.options.append(option)

    def setid(self,id):
        """ 
        Set the id of this style

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: id to assign to this style
        @type id: int
        """

        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the style. In other words
        return the javascript code representing the style.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of style
        @rtype: string
        """
        return "\t\t\tvar style%i = new OpenLayers.Style({%s\n\t\t\t});\n"%(self.id,','.join(["\n\t\t\t\t%s"%str(x) for x in self.options]) if len(self.options)>0 else "")

class Icon:
    """ 
    Wrapper around the OpenLayers Icon object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self,url,size,offset):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param url: URL of icon
        @type url: string
        @param size: Size of icon
        @type size: map.Size
        @param offset: Offset of icon from point 
        @type offset: map.Pixel
        """

        self.url = url
        self.size = size
        self.offset = offset
        self.id = 0
        
    def setid(self,id):
        """ 
        Set the id of this icon

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: id to assign to this icon
        @type id: int
        """

        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the Icon. In other words
        return the javascript code representing the Icon.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Icon
        @rtype: string
        """
        return "\t\t\tvar icon%i = new OpenLayers.Icon(\"%s\",\n\t\t\t\t%s,\n\t\t\t\t%s\n\t\t\t);"%(self.id,self.url,str(self.size),str(self.offset))

class Size:
    """ 
    Wrapper around the OpenLayers Size object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self,x,y):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param x: Horizontal size in pixels
        @type x: int
        @param y: Vertical size in pixels
        @type y: int
        """

        self.x = x
        self.y = y

    def __str__(self):
        """ 
        Return a textual representation of the Size. In other words
        return the javascript code representing the Size.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Size
        @rtype: string
        """

        return "new OpenLayers.Size(%s,%s)"%(self.x,self.y)

class Pixel:
    """ 
    Wrapper around the OpenLayers Pixel object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self,x,y):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param x: Horizontal position of pixel
        @type x: int
        @param y: Vertical position of pixel
        @type y: int
        """

        self.x = x
        self.y = y

    def __str__(self):
        """ 
        Return a textual representation of the Pixel. In other words
        return the javascript code representing the Pixel.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Pixel
        @rtype: string
        """

        return "new OpenLayers.Pixel(%s,%s)"%(self.x,self.y)

class MapOption:
    """ 
    Represents an option for a Map object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, name, value):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param name: Name of option
        @type name: string
        @param value: Value of option
        @type value: string
        """
        self.name = name
        self.value = value

    def __str__(self):
        """ 
        Return a textual representation of the MapOption. In other words
        return the javascript code representing the MapOption.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of MapOption
        @rtype: string
        """
        return "\t\t'%s': %s"%(self.name,self.value)

class Control:
    """ 
    Wrapper around the OpenLayers Control object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, type, options=""):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param type: Type of control
        @type type: string
        @param options: Optional control options
        @type options: string
        """

        self.type = type
        self.options = options
        self.id = 0

    def setid(self,id):
        """ 
        Set the id of this Control

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: Id to assign to Control
        @type id: int
        """

        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the Control. In other words
        return the javascript code representing the Control.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Control
        @rtype: string
        """
        return "new OpenLayers.Control.%s(%s)"%(self.type,self.options)

class Layer:
    """ 
    Wrapper around the OpenLayers Layer object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, type, title, url=""):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param type: Layer type
        @type type: string
        @param title: Layer title
        @type title: string
        @param url: Optional URL of Layer
        @type url: string
        """

        self.title = title
        self.url = url
        self.type = type
        self.options = []
        self.id = 0
        if self.url != "":
            self.url = ",\n\t\t\t\t\"%s\""%self.url

    def addoption(self,option):
        """ 
        Adds an option to the Layer

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param option: Option to be added to the Layer
        @type option: map.LayerOption

        @todo: Remove duplicate options
        """

        self.options.append(option)

    def setid(self,id):
        """ 
        Set the id of the Layer

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: Id to assign to Layer
        @type id: int
        """
        
        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the Layer. In other words
        return the javascript code representing the Layer.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Layer
        @rtype: string
        """

        return "\t\t\tvar layer%i = new OpenLayers.Layer.%s(\n\t\t\t\t\"%s\"%s%s\n\t\t\t);\n"%(self.id,self.type,self.title,self.url,",\n\t\t\t\t{%s}"%','.join([str(x) for x in self.options]) if len(self.options)>0 else "")

class LayerOption:
    """ 
    Represents an Option of a OpenLayers Layer

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, name, value):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param name: Option name
        @type name: string
        @param value: Option value
        @type value: string
        """

        self.name = name
        self.value = value

    def __str__(self):
        """ 
        Return a textual representation of the LayerOption. In other words
        return the javascript code representing the LayerOption.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of LayerOption
        @rtype: string
        """

        return "'%s': %s"%(self.name,self.value)

class Projection:
    """ 
    Wrapper around an OpenLayers Projection object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, projection):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param projection: Projection (beg. EPSG:4326)
        @type projection: string
        """

        self.projection = projection

    def __str__(self):
        """ 
        Return a textual representation of the Projection. In other words
        return the javascript code representing the Projection.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Projection
        @rtype: string
        """

        return "new OpenLayers.Projection(\"%s\")"%self.projection

class Bounds:
    """ 
    Wrapper around the OpenLayers Bounds object

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, bounds, transfrom=None, transto=None):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param bounds: Integer or latitude/longitude of the corners of the bounding box
        @type bounds: (longitudemin, latitudemin, longitudemax, latitudemax)
        @param transfrom: Optional transformation
        @type transfrom: map.Projection
        @param transto: Optional transformation to
        @type transto: map.Projection
        """

        self.lonmin = bounds[0]
        self.latmin = bounds[1]
        self.lonmax = bounds[2]
        self.latmax = bounds[3]
        self.transfrom = transfrom
        self.transto = transto
        self.id = 0

    def setid(self,id):
        """ 
        Set id of Bounds

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param id: Id to assign Bounds
        @type id: int
        """

        self.id = id

    def __str__(self):
        """ 
        Return a textual representation of the Bounds. In other words
        return the javascript code representing the Bounds.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of Bounds
        @rtype: string
        """

        retval = "\t\t\tvar bounds%i = new OpenLayers.Bounds(%s,%s,%s,%s);"%(self.id,self.lonmin,self.latmin,self.lonmax,self.latmax)
        if self.transfrom and self.transto: retval += "\n\t\t\tbounds%i.transform(%s,%s);"%(self.id,str(self.transfrom),str(self.transto))
        return retval

def defaultMap(name, div):
    """ 
    Generate a default OpenLayers map. The following default values will be used:
        - Boundary:              -20037508, -20037508,20037508, 20037508.34
        - Projection:            EPSG:900913
        - Display Projection:    EPSG:4326
        - Units:                 meters
        - Number of zoom levels: 18
        - Maximum resolution:    156543.0339

        - Layers:
            - Maps: 
                - Open Street Maps
                - Open Aerial Maps
                - Tiles@Home
                - Google Street Maps
                - Google Satellite Maps
                - Google Hybrid Maps
                - Virtual Earth Roads
                - Virtual Earth Aerial
                - Yahoo Street
                - Yahoo Satellite
                - Yahoo Hybrid
            - Other:
                - Editable vector layer

        - Controls:
            - Navigation
            - PanZoomBar
            - Attribution
            - LayerSwitcher
            - EditingToolbar            

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param name: Name of javascript variable to represent map
    @type name: string
    @param div: ID of DIV to render map on
    @type div: String

    @return: Map object
    @rtype: Map
    """

    map = Map(name, div)

    bnd = map.addbounds(Bounds((-20037508, -20037508,20037508, 20037508.34)))

    map.addoption(MapOption("projection", Projection("EPSG:900913")))
    map.addoption(MapOption("displayProjection", Projection("EPSG:4326")))
    map.addoption(MapOption("units", "\"m\""))
    map.addoption(MapOption("numZoomLevels", 18))
    map.addoption(MapOption("maxResolution", 156543.0339))
    map.addoption(MapOption("maxExtend", bnd))

    layers = []

    layers.append(Layer("OSM","OpenStreetMaps"))

    layers.append(Layer("XYZ","OpenAerialMaps","http://tile.openaerialmap.org/tiles/1.0.0/openaerialmap-900913/${z}/${x}/${y}.png"))

    layers.append(Layer("OSM","OpenStreetMaps (Tiles@Home)","http://tah.openstreetmap.org/Tiles/tile/${z}/${x}/${y}.png"))

    layers.append(Layer("Google","Google Streets"))

    layers.append(Layer("Google","Google Satellite"))
    layers[4].addoption(LayerOption("type","G_SATELLITE_MAP"))
    layers[4].addoption(LayerOption("numZoomLevels","22"))

    layers.append(Layer("Google","Google Hybrid"))
    layers[5].addoption(LayerOption("type","G_HYBRID_MAP"))

    layers.append(Layer("VirtualEarth","Virtual Earth Roads"))
    layers[6].addoption(LayerOption("type","VEMapStyle.Road"))

    layers.append(Layer("VirtualEarth", "Virtual Earth Aerial"))
    layers[7].addoption(LayerOption("type","VEMapStyle.Aerial"))

    layers.append(Layer("VirtualEarth", "Virtual Earth Hybrid"))
    layers[8].addoption(LayerOption("type", "VEMapStyle.Hybrid"))

    layers.append(Layer("Yahoo", "Yahoo Street"))

    layers.append(Layer("Yahoo", "Yahoo Satellite"))
    layers[10].addoption(LayerOption("type", "YAHOO_MAP_SAT"))

    layers.append(Layer("Yahoo", "Yahoo Hybrid"))
    layers[11].addoption(LayerOption("type", "YAHOO_MAP_HYB"))

    layers.append(Layer("Vector","Editable Vectors"))

    map.addlayer(layers[0])

    for layer in layers[1:11]:
        layer.addoption(LayerOption("sphericalMercator", "true"))
        map.addlayer(layer)

    vectorlayer = map.addlayer(layers[12])

    map.addcontrol(Control("Navigation"))
    map.addcontrol(Control("PanZoomBar"))
    map.addcontrol(Control("Attribution"))
    map.addcontrol(Control("LayerSwitcher"))
    map.addcontrol(Control("EditingToolbar",vectorlayer))

    return map

if __name__=='__main__':
    print str(defaultMap('map','map'))
