#!/usr/bin/env python

""" 
 Represents an OpenLayer map HTML-page. This is not the wrapper around OpenLayers.
 This module only represents the HTML-page. For the wrapper see map.py

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""
 
import map

class MapPage:
    """ 
    Represents an OpenLayer map HTML-page.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, title, body=""):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param title: Title of the page (body of <title>-tag)
        @type title: string
        @param body: The content of body will be added to the page after the map-DIV and before the </html>-tag
        @type body: string
        """

        self.title = title
        self.body = body
        self.stylesheets = []
        self.sources = []
        self.maps = []

    def addstylesheet(self, stylesheet):
        """ 
        Add a stylesheet to the HTML-page.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param stylesheet: Stylesheet to be added to page.
        @type stylesheet: Stylesheet
        """

        self.stylesheets.append(stylesheet)

    def addsource(self, source):
        """ 
        Add a script source to the HTML-page.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param source: Script source to be added to page.
        @type source: JSSource
        @todo: Should by type Source, and JSSource should inherit source
        """

        self.sources.append(source)

    def addmap(self, map, style):
        """ 
        Add a map to the HTML-page.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param map: Map to be added to the page
        @type map: Map
        @param style: Inline style of map-DIV 
        @type style: string
        @todo: Should have width and height as separate parameters instead of part of the style variable
        """

        self.maps.append([map,style])

    def __str__(self):
        """ 
        Return a textual representation of the HTML-page. In other words, return the
        HTML-page as itself.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of HTML-page
        @rtype: string
        """

        return """<html xmlns="http://www.w3.org/1999/xhtml">
\t<head>
\t\t<title>%s</title>
%s
%s
\t\t<script type=\"text/javascript\">
\t\t\tfunction init(){
%s
\t\t\t}
\t\t</script>
\t</head>
\t<body onload="init()">
%s
%s
\t</body>
</html>"""%(self.title,'\n'.join(["\t\t%s"%str(x) for x in self.stylesheets]),'\n'.join(["\t\t%s"%str(x) for x in self.sources]),'\n'.join([str(x[0]) for x in self.maps]),'\n'.join(["\t\t<div id=\"%s\" style=\"%s\"><div>"%(x[0].div,str(x[1])) for x in self.maps]),'\n'.join(["\t\t%s"%x for x in self.body.splitlines()]))

class Stylesheet:
    """ 
    Represents a stylesheet.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, url):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param url: URI of stylesheet to represent
        @type url: string
        """

        self.url = url

    def __str__(self):
        """ 
        Return a textual representation of the stylesheet. In other words, return the
        <link>-tag pointing to the stylesheet.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of stylesheet
        @rtype: string
        """
        
        return "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\" />"%self.url

class JSSource:
    """ 
    Represents a javascript source.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, url):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param url: URI of javascript source
        @type url: string
        @todo: Need to make a Source class, and make JSSource a child of it (see above todo)
        """

        self.url = url

    def __str__(self):
        """ 
        Return a textual representation of the javascript source. In other words, return
        the <script>-tag pointing to the javascript source.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of javascript source.
        @rtype: string
        """

        return "<script src=\"%s\"></script>"%self.url

def defaultpage(title, body=""):
    """ 
    Generate a default map HTML-page, at the moment complete with API-ids for 
    virtualearth, google maps and yahoo maps (should be removed - see todo
    below). For default values of the map itself, please see defaultMap
    method in map.py

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0

    @param title: Title of the map HTML-page (body of the <title>-tag)
    @type title: string
    @param body: The content of body will be added to the page after the map-DIV and before the </html>-tag
    @type body: string

    @return: Default map HTML-page 
    @rtype: MapPage
    @todo: Need to move api keys out into separate file
    """

    page = MapPage(title, body)
    page.addstylesheet(Stylesheet("/lib/theme/default/style.css"))
    page.addsource(JSSource("http://dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.1"))
    page.addsource(JSSource("http://maps.google.com/maps?file=api&amp;v=2&amp;key=ABQIAAAAP4een1Q_B12O6R_uQvPkcBQhrvAM14cAG7Fz7SM6gjF33uubjxTvqYx0-Gmt3Wdi0C-ggdZ4Dz9TtA"))
    page.addsource(JSSource("http://api.maps.yahoo.com/ajaxymap?v=3.0&appid=IJO1uGjV34En9VX39doc1KeylvV2xE4f6jnzAyrEiulTY7f_YZVxpO.iqMbNJKsl6m9DoYtZ"))
    page.addsource(JSSource("/lib/OpenLayers.js"))
    return page

if __name__=='__main__':
    page = defaultpage('Testmap','<p>This is a map page test!</p>')
    page.addmap(map.defaultMap('map','map'),"width:100%;height:100%;")
    print(str(page))
