#!/usr/bin/env python

""" 
 Represents a wfs-xml file containing any number of single points
 and/or filled polygons.
 Used when generating single points and convex hulls for cell
 observations. When generating overviews of cells, please use
 georss instead.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
 @todo: Add options for other types of WFS-features
"""

class WFS:
    """ 
    Represents a wfs-xml page containing any number of single points
    and/or filled polygons.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, site):
        """ 
        Init class

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param site: Base URI of website wfs-xml originates from.
        @type site: string
        """

        self.polycounter = 0
        self.items = []
        self.header = """<?xml version="1.0" encoding="utf-8"?>
<wfs:FeatureCollection xmlns:ms="%s" 
xmlns:wfs="http://www.opengis.net/wfs" 
xmlns:gml="http://www.opengis.net/gml" 
xmlns:ogc="http://www.opengis.net/ogc" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
xsi:schemaLocation="http://schemas.opengis.net/wfs/1.0.0/WFS-basic.xsd">\n"""%site

    def addboundedby(self, bounds):
        """ 
        Add a bounding box to the wfs-xml

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param bounds: extreme points of bounding box
        @type bounds: tupple of coordinates
        """

        self.items.append("""<gml:boundedBy>
      	<gml:Box srsName="EPSG:4326">
      		<gml:coordinates>%s,%s %s,%s</gml:coordinates>
      	</gml:Box>
      </gml:boundedBy>\n"""%bounds)

    def addpoint(self,longitude,latitude):
        """ 
        Add a single point to the wfs-xml.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param longitude: Longitude of point
        @type longitude: string
        @param latitude: Latitude of point
        @type latitude: string
        """

        self.items.append("""<gml:featureMember>
      <gml:Point>
	<gml:coordinates>%s,%s</gml:coordinates>
      </gml:Point>
    </gml:featureMember>\n"""%(longitude,latitude))

    def addfilledpoly(self,ul,ll,bounds,name):
        """ 
        Add a filled polygon to the wfs-xml.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param ul: Upper points
        @type ul: list
        @param ll: Lower points
        @type ll: list
        @param bounds: Boundries of polygon
        @type bounds: tupple
        @param name: Name of feature
        @type name: string
        """

        self.polycounter += 1
        bnds = "%s,%s %s,%s"%bounds 
        ll.reverse()
        poly = "%s\n%s"%(" ".join(["%s,%s"%(x[0],x[1]) for x in ul])," ".join(["%s,%s"%(x[0],x[1]) for x in ll]))
        self.items.append("""<gml:featureMember>
<ms:polygon>
<gml:boundedBy>
<gml:Box srsName="EPSG:4326">
<gml:coordinates>
%s
</gml:coordinates>
</gml:Box>
</gml:boundedBy>
<ms:msGeometry>
<gml:Polygon srsName="EPSG:4326">
<gml:outerBoundaryIs>
<gml:LinearRing>
<gml:coordinates>
%s
</gml:coordinates>
</gml:LinearRing>
</gml:outerBoundaryIs>
</gml:Polygon>
</ms:msGeometry>
<ms:ogc_fid>
%i
</ms:ogc_fid>
<ms:name>
%s
</ms:name>
</ms:polygon>
</gml:featureMember>"""%(bnds,poly,self.polycounter,name))

    def __str__(self):
        """ 
        Return a textual representation of the wfs-xml. In other words
        return the xml itself.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: Textual representation of wfs-xml
        @rtype: string
        """
        
        retval = self.header
        for item in self.items:
            retval += item
        retval += "</wfs:FeatureCollection>"
        return retval
