#!/usr/bin/env python
""" 
 The XML-RPC backend queryfilter module. See XML-RPC backend documentation 
 for more information.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import logging
import logging.config
import glocdb
import re

class Error(Exception):
    """ 
    Base class for exceptions in this module.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    pass

class FilterNotFoundError(Error):
    """ 
    Exception used to indicate that the requsted filter doesn't exist.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self):
        """ 
        Initialize FilterNotFoundError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = "Provided filter does not exist"

    def __str__(self):
        """ 
        String representation of FilterNotFoundError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class FilterNotValidError(Error):
    """ 
    Exception used to indicate that the requsted filter doesn't have required run method.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self):
        """ 
        Initialize FilterNotvalidError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = "Provided filter exists, but is not a valid filter."

    def __str__(self):
        """ 
        String representation of FilterNotValidError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class QueryFilter:
    """ 
    Query Filter module class.
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, imei, fingerprints, filter, conf):
        """ 
        Initialize QueryFilter

        See L{server.GlocHandler.xmlrpc_locationQuery} for detailed information about the format of
        the fingerprints variable.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting query
        @type imei: str
        @param fingerprints: fingerprint(s) to be used for query
        @type fingerprints: list
        @param filter: name of filter to be used for query
        @type filter: str
        @param conf: server configuration containing among others database information
        @type conf: L{ServerConf}

        @raise FilterNotFoundError: the requested filter does not exist
        @raise FilterNotValidError: the filter does not contain a run-method


        @todo: Should also check that self.filter actually is of type filter
        """
        self.imei = imei
        self.fingerprints = fingerprints
        self.logger = logging.getLogger("query")
        self.db = glocdb.GlocDB(conf)

        if not re.match(r'^[a-zA-Z0-9]+$',filter):
            raise FilterNotFoundError()

        try:
            exec "import queryfilters.%s as filters"%filter
        except:
            raise FilterNotFoundError()

        self.filter = filters.Filter(imei, fingerprints, self.db)

        if not callable(getattr(self.filter, "run", None)):
            raise FilterNotValidError()

    def run(self):
        """ 
        Run this query and return result

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: result
        @rtype: list

        #raise FilterNotValidError: The filter returned a non-valid result set
        """

        self.logger.info("Sent fingerprint(s) to filter: %s"%self.fingerprints)
        result = self.filter.run()
        self.logger.info("Got result from filter: %s"%result)
        if not self.__validResult__(result):
            raise FilterNotValidError()
        return result
    
    def __validResult__(self, result):
        """ 
        Checks if a result from a filter is a valid result. As of now, a valid result
        consists of a dict with the boolean parameter status (indicating if the result
        actually contains a location). If status is True, then the result must at least
        contain latitude and longitude parameters. Several other optional parameters
        can be used (see the main gloc xml-rpc documentation).

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param result: result to be checked
        @type result: dict

        @return: valid result
        @rtype: bool
        """

        if type(result).__name__!='dict': return False
        if not 'status' in result: return False
        if type(result['status']).__name__!='bool': return False
        if result['status']:
            if not 'latitude' in result: return False
            if not 'longitude' in result: return False
        return True
