""" 
 Server configuration object for the gloc XML-RPC backend server.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import os
import ConfigParser
import fileUtils
from serverConst import *

class ServerConf():

    """ 
    Server configuration class.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def get_configpath(self):
        """ 
        Returns the full path to the configuration file in use

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: full path to configuration file
        @rtype: str
        """

        configpath = os.path.expanduser(DEFAULT_PATH)
        fileUtils.check_dir(configpath)
        configpath = os.path.join(configpath, 'glocalizer-server.conf')
        return configpath

    def __init__(self):
        """ 
        Initialize ServerConf

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        configpath = self.get_configpath()
        self.read(configpath)
        if not os.path.exists(configpath):
            self.write(configpath)

    def write(self, configpath):
        """ 
        Write configuration file to disk.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param configpath: full path to write configuration file to
        @type configpath: str
        """

        config = ConfigParser.RawConfigParser()
        config.add_section(SECTION_INIT)
        config.add_section(SECTION_DATABASE)
        if self.init_path:
            config.set(SECTION_INIT, 'path', self.init_path)
        config.set(SECTION_INIT, 'queryfilter', self.default_query_filter)
        config.set(SECTION_INIT, 'updatefilter', self.default_update_filter)
        config.set(SECTION_INIT, 'filterchange', self.filter_change_allowed)
        config.set(SECTION_INIT, 'secret', self.secret)
        config.set(SECTION_INIT, 'filterpath', self.filterpath)
        config.set(SECTION_DATABASE, 'host', self.db_host)
        config.set(SECTION_DATABASE, 'port', self.db_port)
        config.set(SECTION_DATABASE, 'name', self.db_name)
        config.set(SECTION_DATABASE, 'username', self.db_username)
        config.set(SECTION_DATABASE, 'password', self.db_password)

        configfile = open(configpath, 'wb')
        config.write(configfile)

    def read(self, configpath):
        """ 
        Read a configuration file from disk.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param configpath: full path to configuration file
        @type configpath: str
        """

        def read_config(section, keyOption, defaultValue, castFunction):
            try:
                strValue = config.get(section, keyOption)
                return castFunction(strValue)
            except Exception:
                return defaultValue

        config = ConfigParser.RawConfigParser()
        config.read(configpath)

        self.init_path = os.path.join(os.path.expanduser(USER_PATH), CONFIG_PATH)
        strPath = read_config(SECTION_INIT,'path', self.init_path, str)
        if not strPath.strip().lower() in ['none', '']:
            strPath = fileUtils.check_dir(strPath)
            if os.path.isdir(strPath):
                self.init_path = strPath

        self.default_query_filter = read_config(SECTION_INIT,'queryfilter', 'dummy', str)
        self.default_update_filter = read_config(SECTION_INIT,'updatefilter', 'dummy', str)
        self.filter_change_allowed = read_config(SECTION_INIT,'filterchange', True, bool)
        self.secret = read_config(SECTION_INIT,'secret','themostsecretstringever',str)
        self.filterpath = read_config(SECTION_INIT,'filterpath',os.path.join(os.environ['HOME'], 'studier/masterproject/gloc/py', 'filters'),str)

        self.db_host = read_config(SECTION_DATABASE,'host','localhost',str)
        self.db_port = read_config(SECTION_DATABASE,'port',5432,int)
        self.db_name = read_config(SECTION_DATABASE,'name','gloc',str)
        self.db_username = read_config(SECTION_DATABASE,'username','gloc',str)
        self.db_password = read_config(SECTION_DATABASE,'password','99ab97a',str)

    def save(self):
        """ 
        Write configuration file to default location on disk.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """

        configpath = self.get_configpath()
        self.write(configpath)

