#!/usr/bin/env python
""" 
 The XML-RPC backend update module. See XML-RPC backend documentation 
 for more information.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

from updatefilter import UpdateFilter
import logging
import logging.config

class Error(Exception):
    """ 
    Base class for exceptions in this module.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    pass

class BogusIMEIError(Error):
    """ 
    Exception used to indicate non valid IMEI.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self):
        """ 
        Initialize BousIMEIError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = "Provided IMEI was illegal"

    def __str__(self):
        """ 
        String representation of BousIMEIError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class InvalidFingerprintError(Error):
    """ 
    Exception used to indicate non valid fingerprint(s).

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self, value):
        """ 
        Initialize InvalidFingerprintError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = value

    def __str__(self):
        """ 
        String representation of InvalidFingerprintError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """

        return repr(self.value)

class InvalidGPSError(Error):
    """ 
    Exception used to indicate non valid GPS data.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self, value):
        """ 
        Initialize InvalidGPSError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = value

    def __str__(self):
        """ 
        String representation of InvalidGPSError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class Update:
    """ 
    Update module class.
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, imei, fingerprints, gps, filter, conf):
        """ 
	Initialize Update

        See L{server.GlocHandler.xmlrpc_locationUpdate} for detailed information about the format of
        the fingerprints and gps variables.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting update
        @type imei: str
        @param fingerprints: Fingerprint(s) that should be used for this update
        @type fingerprints: list
        @param gps: GPS information that should be used for this update
        @type gps: dict
        @param filter: filter that should be used for this update
        @type filter: str
        @param conf: server configuration containing among others database information
        @type conf: L{ServerConf}

        @raise BogusIMEIError: Supplied IMEI is incorrectly formated
        @raise InvalidFingerprintError: Supplied fingerprint(s) are incorrectly formated
        @raise InvalidGPSError: Supplied GPS data is incorrectly formated
        """

        self.imei = imei
        self.fingerprints = fingerprints
        self.gps = gps
        self.filter = filter
        self.logger = logging.getLogger("update")
        self.conf = conf

        if not self.validIMEI(imei):
            raise BogusIMEIError()

        if not self.validFingerprints(fingerprints):
            raise InvalidFingerprintError("Invalid fingerprint(s)")

        if not self.validGPS(gps):
            raise InvalidGPSError('Invalid gps data')

    def run(self):
        """ 
        Run this update and return the result.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: result of update
        @rtype: list
        """

        filter = UpdateFilter(self.imei, self.fingerprints, self.gps, self.filter, self.conf)
        return filter.run()

    def validIMEI(self, imei):
        """ 
        Checks if the supplied IMEI is valid according to specifications in
        http://www.3gpp.org/ftp/Specs/html-info/23003.htm

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI number to be checked
        @type imei: str

        @return: True if legal IMEI, else False
        @rtype: bool

        @todo: this function should really be a part of a general API to avoid duplicates
        @todo: this function can be much more advanced... chekcing among others check digit. see http://en.wikipedia.org/wiki/Imei
        @todo: we can also get IMEI info for individual imeis from http://www.numberingplans.com/?page=analysis
        """

        try:
            if int(imei)>0 and int(imei)<9999999999999999:
                return True
            return False
        except:
            return False

    def validGPS(self, gps):
        """ 
        Check if the supplied GPS-data is valid and contains at least the mandatory fields

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param gps: GPS-data to be inspected
        @type gps: dict

        @return: True if valid GPS-data, else False
        @rtype: bool

        @todo: This function should really ba a part of a general API to avoid duplicates
        @todo: Should other fields be mandatory?
        """

        if type(gps).__name__!='dict': return False
        if not 'latitude' in gps: return False
        if not 'longitude' in gps: return False
        return True

    def validFingerprints(self, fingerprints):
        """ 
        Check if the supplied fingerprint(s) is valid and contains at least the mandatory fields for its type.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param fingerprints: list of fingerprint(s) to be inspected
        @type fingerprints: list

        @return: True if fingerprint(s) are valid, else False
        @rtype: bool

        @todo: This function should really be a part of a general API to avoid duplicates
        """

        if type(fingerprints).__name__!='list': return False
        if len(fingerprints)<1: return False
        for fingerprint in fingerprints:
            if type(fingerprint).__name__!='dict': return False
            if not 'type' in fingerprint: return False
            if fingerprint['type']!='gsm' and fingerprint['type']!='wlan': return False
            if fingerprint['type']=='gsm':
                if not 'cellid' in fingerprint: return False
                if not 'la' in fingerprint: return False
                if not 'mnc' in fingerprint: return False
                if not 'mcc' in fingerprint: return False
                if not 'rxlev' in fingerprint: return False
            else:
                if not 'ssid' in fingerprint: return False
                if not 'rxlev' in fingerprint: return False
        return True
