#!/usr/bin/env python
""" 
 The XML-RPC backend updatefilter module. See XML-RPC backend documentation 
 for more information.

 @author: Brendan Johan Lee
 @contact: brendajl@simula.no

 @version: 1.0
"""

import logging
import logging.config
import glocdb
import re

class Error(Exception):
    """ 
    Base class for exceptions in this module.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    pass

class FilterNotFoundError(Error):
    """ 
    Exception used to indicate that the requsted filter doesn't exist.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self):
        """ 
        Initialize FilterNotFoundError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = "Provided filter does not exist"

    def __str__(self):
        """ 
        String representation of FilterNotFoundError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class FilterNotValidError(Error):
    """ 
    Exception used to indicate that the requsted filter doesn't have required run method.

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """
    def __init__(self):
        """ 
        Initialize FilterNotvalidError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.value = "Provided filter exists, but is not a valid filter."

    def __str__(self):
        """ 
        String representation of FilterNotValidError

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: string representation
        @rtype: str
        """
        return repr(self.value)

class UpdateFilter:
    """ 
    Update Filter module class.
    
    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self, imei, fingerprints, gps, filter, conf):
        """ 
        Initialize UpdateFilter

        See L{server.GlocHandler.xmlrpc_locationUpdate} for detailed information about the format of
        the fingerprints and gps variables.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting update
        @type imei: str
        @param fingerprints: fingerprint(s) to be used for update
        @type fingerprints: list
        @param gps: GPS-data to be used for update
        @type gps: dict
        @param filter: name of filter to be used for update
        @type filter: str
        @param conf: server configuration containing among others database information
        @type conf: L{ServerConf}

        @raise FilterNotFoundError: the requested filter does not exist
        @raise FilterNotValidError: the filter does not contain a run-methodn

        @todo: We could do an IMEI check here, so as to only allow white or blacklisted IMEIs or something
        @todo: Should also check that self.filter actually is of type filter
        """

        self.imei = imei
        self.fingerprints = fingerprints
        self.gps = gps
        self.logger = logging.getLogger("update")
        self.db = glocdb.GlocDB(conf)

        if not re.match(r'^[a-zA-Z0-9]+$',filter):
            raise FilterNotFoundError()
        
        try:
            exec "import updatefilters.%s as filters"%filter
        except:
            raise FilterNotFoundError()

        self.filter = filters.Filter(imei, fingerprints, gps, self.db)

        if not callable(getattr(self.filter, "run", None)):
            raise FilterNotValidError()


    def run(self):
        """ 
        Run this update and return result

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @return: result
        @rtype: list

        @todo: Enter data in database
        @todo: Should not return result of query, but something else
        """

        self.logger.info("Sent fingerprint(s) and gps to filter: %s %s"%(self.fingerprints,self.gps))
        result = self.filter.run()
        self.logger.info("Got result from filter: %s"%result)
        return result
