#!/usr/bin/env python
""" 
Main file of the Gloc XML-RPC backend server. Based on the twisted framework. 
(apt-get install python-twisted on Debian-based systems)

@author: Brendan Johan Lee
@contact: brendajl@simula.no
@version: 1.0
 
@todo: Make install and usage documentation for all of my different mastersproject apps
@todo: Collect all documentation we have of all sorts in a webpage
@todo: Make documentation for the gui app
@todo: Allow GUI-app to load overlays from file so we can display results
@todo: make sure all files are in the api-doc generator
@todo: In the def and class py - snippets we should suppress the selv param from being documented
"""
## GNU General Public Licence (GPL)
 
## This program is free software; you can redistribute it and / or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 2 of the License,  or (at your option) any later
## version.
## This program is distributed in the hope that it will be useful,  but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
## FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
## details.
## You should have received a copy of the GNU General Public License along with
## this program; if not,  write to the Free Software Foundation,  Inc.,  59 Temple
## Place,  Suite 330,  Boston,  MA  02111 - 1307  USA

from twisted.web import xmlrpc, server
from twisted.internet import reactor
import gloclibs.server.query.query as query
import gloclibs.server.query.queryfilter as queryfilter
import gloclibs.server.update.update as update
import gloclibs.server.update.updatefilter as updatefilter
from gloclibs.server.serverConf import ServerConf

import logging
import logging.config
import sys

class GlocHandler(xmlrpc.XMLRPC):
    """ 
    XML-RPC class to handle incoming requests

    @author: Brendan Johan Lee
    @contact: brendajl@simula.no

    @version: 1.0
    """

    def __init__(self):
        """ 
        Initialize GlocHandler

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0
        """
        self.conf = ServerConf()
        sys.path.append(self.conf.filterpath)
        xmlrpc.XMLRPC.__init__(self)

    def xmlrpc_locationUpdate(self, imei, fingerprints, gps, filter=None):
        """ 
        Method that handles locationUpdate calls to the XML-RPC server.

        Fingerprint(s)
        ==============
          The fingerprints variable must be list containing one or more fingerprints, each represented
          by a dict. The dict-entry I{type} is mandatory and must be either 'I{gsm}' or 'I{wlan}'. For
          each of these the following dict-entries are mandatory:

          gsm
          ---
            - cellid - I{Cell ID}
            - mcc - I{Mobile Country Code}
            - mnc - I{Mobile Network Code}
            - la - I{Location Area}
            - rxlev - I{Radio Receive Level}

          wlan
          ----
            - bssid - I{Basic Service Set IDentifier}
            - rxlev - I{Radio Receive Level}

          Many other dict-entries are possible. Please see the XML-RPC backend documentation.

        GPS
        ===
          The gps variable must be a dict, and should contain at least the following entries:
            - latitude
            - longitude
          
          Many other entries are possible. Please see the XML-RPC backend documentation.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param imei: IMEI of device requesting service
        @type imei: str
        @param fingerprints: List of fingerprint(s) (see above).
        @type fingerprints: list
        @param gps: GPS data to be linked with the fingerprint(s)
        @type gps: dict
        @param filter: (Optional) Force server to use different filter. Only works if filter_change_allowed.
        @type filter: str

        @return: Status information
        @rtype: str
        """
        logger = logging.getLogger("update")
        
        self.filter = filter
        if not filter or not self.conf.filter_change_allowed:
            logger.info("Filter forced to default")
            self.filter = self.conf.default_update_filter
        logger.info("Filter in use: %s"%self.filter)
        try:
            u = update.Update(imei,fingerprints,gps,self.filter,self.conf)
            return u.run()
        except update.BogusIMEIError:
            logger.error("Supplied IMEI not valid")
            raise xmlrpc.Fault(101, 'Supplied IMEI not valid')
        except update.InvalidFingerprintError:
            logger.error("Supplied fingerprint(s) raised an error")
            raise xmlrpc.Fault(102, 'Supplied fingerprint(s) raised an error')
        except updatefilter.FilterNotFoundError:
            logger.error("Supplied filter doesn\'t exist")
            raise xmlrpc.Fault(103, 'Supplied filter doesn\'t exist')
        except update.InvalidGPSError:
            logger.error("Supplied GPS data not valid")
            raise xmlrpc.Fault(104, 'Supplied GPS data not valid')
        except updatefilter.FilterNotValidError:
            logger.error("Supplied filter exists, but is not valid")
            raise xmlrpc.Fault(105, 'Supplied filter exists, but is not valid')

    xmlrpc_locationUpdate.signature = [['list', 'str', 'list', 'dict', 'str']]

    def xmlrpc_locationQuery(self, imei, fingerprints, filter=None, gps=None):
        """ 
        Method that handles locationQuery calls to the XML-RPC server.

        Fingerprint(s)
        ==============
          The fingerprints variable must be list containing one or more fingerprints, each represented
          by a dict. The dict-entry I{type} is mandatory and must be either 'I{gsm}' or 'I{wlan}'. For
          each of these the following dict-entries are mandatory:

          gsm
          ---
            - cellid - I{Cell ID}
            - mcc - I{Mobile Country Code}
            - mnc - I{Mobile Network Code}
            - la - I{Location Area}
            - rxlev - I{Radio Receive Level}

          wlan
          ----
            - bssid - I{Basic Service Set IDentifier}
            - rxlev - I{Radio Receive Level}

          Many other dict-entries are possible. Please see the XML-RPC backend documentation.

        GPS
        ===
          The gps variable is optional and is not used to calculate location. The content of 
          the variable is simply saved in the log for future reference.
          The gps variable must be a dict, and should contain at least the following entries:
            - latitude
            - longitude
          
          Many other entries are possible. Please see the XML-RPC backend documentation.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0

        @param imei: IMEI of device requesting service
        @type imei: str
        @param fingerprints: List of fingerprint(s) (see above).
        @type fingerprints: list
        @param gps: Optional GPS-data. Will be stored in log file for future reference.
        @type gps: dict
        @param filter: (Optional) Force server to use different filter. Only works if filter_change_allowed.
        @type filter: str

        @return: estimated location
        @rtype: dict

        @todo: remove the gps variable
        @todo: actually make a good return data type... and document it here
        """
        logger = logging.getLogger("query")
        
        self.filter = filter
        if not filter or not self.conf.filter_change_allowed:
            logger.info("Filter forced to default")
            self.filter = self.conf.default_query_filter
        logger.info("Filter in use: %s"%self.filter)
        if gps:
            logger.info("We also got gps: %s"%gps)
        try:
            q = query.Query(imei,fingerprints,self.filter,self.conf)
            return q.run()
        except query.BogusIMEIError:
            logger.error("Supplied IMEI not valid")
            raise xmlrpc.Fault(101, 'Supplied IMEI not valid')
        except query.InvalidFingerprintError:
            logger.error("Supplied fingerprint(s) raised an error")
            raise xmlrpc.Fault(102, 'Supplied fingerprint(s) raised an error')
        except queryfilter.FilterNotFoundError:
            logger.error("Supplied filter not usable")
            raise xmlrpc.Fault(103, 'Supplied filter not usable')
        except queryfilter.FilterNotValidError:
            logger.error("Supplied filter exists, but is not valid")
            raise xmlrpc.Fault(105, 'Supplied filter exists, but is not valid')

    xmlrpc_locationQuery.signature = [['list', 'str', 'list', 'str']]

    def xmlrpc_setDefaultUpdateFilter(self, filter, secret):
        #TODO: Check if secret is valid, and if so change servers default filter
        """ 
        Used to remotely change the default update filter of the server. Checks 
        that secret key is valid, before enforcing change.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param filter: name of filter to make default
        @type filter: str
        @param secret: secret key
        @type secret: str

        @return: success 
        @rtype: bool

        @todo: make this method
        """

        raise NotImplementedError()

    def xmlrpc_setDefaultQueryFilter(self, filter, secret):
        #TODO: Check if secret is valid, and if so change servers default filter
        """ 
        Used to remotely change the default query filter of the server. Checks 
        that secret key is valid, before enforcing change.

        @author: Brendan Johan Lee
        @contact: brendajl@simula.no

        @version: 1.0

        @param filter: name of filter to make default
        @type filter: str
        @param secret: secret key
        @type secret: str

        @return: success 
        @rtype: bool

        @todo: make this method
        """

        raise NotImplementedError()

    def xmlrpc_reload_filters(self):
        """
        Reload any loaded filters. Should be called any time existing filters are
        changes while the server is running.
        
        @author: Brendan Johan Lee
        @contact: brendajl@simula.no
        @version: 1.0
        """
        for module in sys.modules.keys():
            if module.startswith('queryfilters.') or module.startswith('updatefilters.'):
                if type(sys.modules[module]).__name__ == 'module': 
                    reload(sys.modules[module])
        return True



if __name__ == '__main__':
    logging.config.fileConfig("server-logging.conf")
    logging.getLogger("query").info("Starting server...")
    logging.getLogger("update").info("Starting server...")
    r = GlocHandler()
    xmlrpc.addIntrospection(r)
    reactor.listenTCP(7080, server.Site(r))
    reactor.run()
